@@ -23,6 +23,9 @@ Boston, MA 02111-1307, USA.
 #include "util.h"
 #include <libgnomeui/libgnomeui.h>
 
+static GtkWidget *new_folder_dialog;
+static GtkWidget *new_folder_entry;
+
 /* Move the files off the Jukebox and into the harddisk directory */
 static GCallback data_from_jukebox_to_hd (GtkButton *button,
 					  gpointer data)
@@ -158,19 +161,20 @@ static GCallback hdmenu_transfer_response(gpointer data)
   data_from_hd_to_jukebox(NULL, NULL);
 }
 
-
 /* Response to the "delete" call from the harddisk popup menu */
 static GCallback hdmenu_delete_response(gpointer data)
 {
-  GList *metalist = get_all_metadata_from_selection(HDDATA_LIST);
-
-  delete_files(metalist);
-  remove_selected(HDDATA_LIST);
-  destroy_metalist(metalist);
-  // fill_in_dir(HDDATA_LIST, get_current_dir());
+  if (request_confirmation_dialog(_("Really delete selected files?"))) {
+    GList *metalist = get_all_metadata_from_selection(HDDATA_LIST);
+    
+    delete_files(metalist);
+    remove_selected(HDDATA_LIST);
+    destroy_metalist(metalist);
+    // fill_in_dir(HDDATA_LIST, get_current_dir());
+  }
 }
 
-/* Handles descending into a directory in the clist */
+/* Handles descending into a directory */
 static void go_down()
 {
   gchar *path;
@@ -199,6 +203,32 @@ static void go_down()
   destroy_metadata_t(meta);
 }
 
+static void go_down_jukebox()
+{
+  gchar *path;
+  metadata_t *meta;
+
+  meta = get_first_metadata_from_selection(JBDATA_LIST);
+  if (meta == NULL) {
+    return;
+  }
+
+  /* Get the size column. If this is zero, 
+   * we're dealing with a directory! Only
+   * directories of size 0 are allowed, no
+   * files of size 0.
+   */
+  if (meta->size == 0) {
+    GtkWidget *entry = data_widgets.jbentry;
+    
+    // g_print("Changing directory to %s\n", meta->folder);
+    rebuild_datafile_list(meta->folder);
+    // Set the text in the box
+    gtk_entry_set_text (GTK_ENTRY(entry), meta->folder);
+  }
+  destroy_metadata_t(meta);
+}
+
 /* If we come here, then the user has selected a row in the 
  * harddisk file listing */
 static GCallback hdlist_mouseevent(GtkWidget      *widget,
@@ -248,14 +278,65 @@ static GCallback jbmenu_transfer_response(gpointer data)
 }
 
 
+static GCallback create_new_folder_ok_button (GtkButton *button,
+						gpointer data)
+{
+  gchar *foldername;
+
+  foldername = g_strdup(gtk_entry_get_text(GTK_ENTRY(new_folder_entry)));
+  gtk_widget_destroy(new_folder_dialog);
+  // Create it on the jukebox
+  g_print("Creating folder \"%s\"...\n", foldername);
+  jukebox_create_folder(foldername);
+  g_free(foldername);
+}
+
+
+/* Response to the "create folder" call from the harddisk popup menu */
+static GCallback jbmenu_create_folder_response(gpointer data)
+{
+  GtkWidget *label, *button;
+
+  new_folder_dialog = gnome_dialog_new(_("Create a new folder"),
+				       GNOME_STOCK_BUTTON_OK,
+				       GNOME_STOCK_BUTTON_CANCEL,
+				       NULL);
+  gnome_dialog_button_connect(GNOME_DIALOG(new_folder_dialog),
+			      0,
+			      GTK_SIGNAL_FUNC(create_new_folder_ok_button),
+			      NULL);
+  gnome_dialog_button_connect_object(GNOME_DIALOG(new_folder_dialog),
+				     1,
+				     GTK_SIGNAL_FUNC(gtk_widget_destroy),
+				     GTK_OBJECT(new_folder_dialog));
+  gtk_window_set_position (GTK_WINDOW(new_folder_dialog), GTK_WIN_POS_MOUSE);
+
+  label = gtk_label_new(_("Choose a name for the new folder:"));
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show(label);
+  new_folder_entry = gtk_entry_new();
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), new_folder_entry, TRUE, TRUE, 0);
+  gtk_widget_show(new_folder_entry);
+
+  g_signal_connect(G_OBJECT (new_folder_dialog), 
+		   "delete_event",
+		   G_CALLBACK(gtk_widget_destroy),
+		   G_OBJECT (new_folder_dialog));
+
+  gtk_widget_show_all(new_folder_dialog);
+}
+
+
 /* Response to the "delete" call from the jukebox popup menu */
 static GCallback jbmenu_delete_response(gpointer data)
 {
-  GList *metalist = get_all_metadata_from_selection(JBDATA_LIST);
-
-  jukebox_delete_files(metalist);
-  remove_selected(JBDATA_LIST);
-  destroy_metalist(metalist);
+  if (request_confirmation_dialog(_("Really delete selected files?"))) {
+    GList *metalist = get_all_metadata_from_selection(JBDATA_LIST);
+    
+    jukebox_delete_files(metalist);
+    remove_selected(JBDATA_LIST);
+    destroy_metalist(metalist);
+  }
 }
 
 
@@ -265,7 +346,10 @@ static GCallback jblist_mouseevent(GtkWidget      *clist,
 				   GdkEventButton *event,
 				   gpointer        data )
 {
-  if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+  if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
+    go_down_jukebox();
+    return (gpointer) 1;
+  } else if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
     GdkEventButton *bevent = (GdkEventButton *) event;
     
     gtk_menu_popup (GTK_MENU (data_widgets.jukeboxpopupmenu),
@@ -390,6 +474,7 @@ GtkWidget *create_data_widgets(void)
   GtkWidget *panel;
   GtkWidget *transfer_button_vbox;
   GtkWidget *combo;
+  GtkWidget *jbentry;
   GtkWidget *arrow;
   GtkWidget *left_vbox;
   GtkWidget *right_vbox;
@@ -445,6 +530,14 @@ GtkWidget *create_data_widgets(void)
 			  NULL,
 			  0);
   gtk_widget_show (popupmenu_item);
+  popupmenu_item  = gtk_menu_item_new_with_label (_("Create folder"));
+  gtk_menu_shell_append (GTK_MENU_SHELL(popupmenu), popupmenu_item);
+  g_signal_connect_object(GTK_OBJECT(popupmenu_item),
+			  "activate",
+			  G_CALLBACK(jbmenu_create_folder_response),
+			  NULL,
+			  0);
+  gtk_widget_show (popupmenu_item);
   popupmenu_item  = gtk_menu_item_new_with_label (_("Transfer selected"));
   gtk_menu_shell_append (GTK_MENU_SHELL(popupmenu), popupmenu_item);
   g_signal_connect_object(GTK_OBJECT(popupmenu_item),
@@ -573,6 +666,13 @@ GtkWidget *create_data_widgets(void)
   label = gtk_label_new(_("Jukebox data file window"));
   gtk_box_pack_start(GTK_BOX(right_vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
+  /* An entry for showing jukebox directory */
+  jbentry = gtk_entry_new();
+  data_widgets.jbentry = jbentry;
+  gtk_box_pack_start(GTK_BOX(right_vbox), jbentry, FALSE, FALSE, 0);
+  gtk_entry_set_text (GTK_ENTRY(jbentry), "\\");
+  gtk_widget_show (jbentry);
+  /* Then the rest of the stuff... */
   gtk_box_pack_start(GTK_BOX(right_vbox), scrolled_window, TRUE, TRUE, 0);
   gtk_paned_pack2 (GTK_PANED(panel), right_vbox, TRUE, TRUE);
   gtk_widget_show (right_vbox);
