@@ -21,11 +21,6 @@ Boston, MA 02111-1307, USA.
  * disturbs the id3.cpp file */
 #include <libgnomeui/libgnomeui.h>
 
-//#include <libgnomeui-2.0/libgnomeui/gnome-dialog.h>
-//#include <libgnomeui-2.0/libgnomeui/gnome-dialog-util.h>
-//#include <libgnomeui-2.0/libgnomeui/gnome-messagebox.h>
-
-
 #include "filesystem.h"
 #include "jukebox.h"
 #include "xfer.h"
@@ -44,6 +39,7 @@ static gchar *genre;
 static gchar *length;
 static gchar *trackno;
 static gchar *origfname;
+static gchar *folder;
 static gchar *protect;
 
 /* Response to DELETE event (window destruction) */
@@ -104,6 +100,8 @@ static gchar *metalist_get_column(GList *metalist, gint column)
     case TRACKLIST_ID_COLUMN:
       /* Only for TRACKLIST */
       return ret_nonull(meta->path);
+    case TRACKLIST_FOLDER_COLUMN:
+      return ret_nonull(meta->folder);
     default:
       return NULL;
     }
@@ -201,6 +199,13 @@ static void edit_ok_click ( GtkButton *button,
       g_free(newmeta->filename);
       newmeta->filename = g_strdup(origfname);
     }
+    if (folder != NULL &&
+	strlen(folder) &&
+	nullcmp(folder, meta->folder)) {
+      changed = TRUE;
+      g_free(newmeta->folder);
+      newmeta->folder = g_strdup(folder);
+    }
     if (year != NULL &&
 	strlen(year) &&
 	string_to_guint(year) != meta->year) {
@@ -518,6 +523,29 @@ static void edit_dialog_create()
     gtk_widget_show (hbox);
   }
 
+  /* If there was only one file selected we may show the folder name */
+  if (g_list_length(metalist) == 1 && get_prefs_extended_metadata()) {
+    hbox = gtk_hbox_new (FALSE, 0);
+    label = gtk_label_new(_("Folder:"));
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+    gtk_widget_show (label);
+    add_empty_hbox(hbox);
+    entry = gtk_entry_new();
+    gtk_entry_set_width_chars(GTK_ENTRY(entry), 45);
+    folder = metalist_get_column(metalist, TRACKLIST_FOLDER_COLUMN);
+    gtk_entry_set_text(GTK_ENTRY(entry), folder);
+    g_free(folder);
+    folder = (gchar *) gtk_entry_get_text(GTK_ENTRY(entry));
+    g_signal_connect(GTK_OBJECT(entry), 
+		     "changed",
+		     G_CALLBACK(entry_edit_callback),
+		     (gpointer) &folder);
+    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, TRUE, 0);
+    gtk_widget_show(entry);
+    add_to_dialog(dialog , hbox);
+    gtk_widget_show (hbox);
+  }
+
   /* Protection */
   if (g_list_length(metalist) == 1) {
     GtkWidget *checkbox = gtk_check_button_new_with_label(_("Protected"));
@@ -636,6 +664,7 @@ static void prepare_dialog(guint listtype)
   length = NULL;
   trackno = NULL;
   origfname = NULL;
+  folder = NULL;
   protect = NULL;
   if (metadatas == NULL)
     return;
