@@ -25,7 +25,11 @@ Boston, MA 02111-1307, USA.
 #include "mp3file.h"
 #include "id3read.h"
 #include "wmaread.h"
+#include "wavfile.h"
 #include "xfer.h"
+#if !GTK_CHECK_VERSION(2,4,0)
+#include <libgnomeui/libgnomeui.h>
+#endif
 
 /* Extra includes used by this file only */
 
@@ -36,6 +40,9 @@ Boston, MA 02111-1307, USA.
 #include <sys/stat.h>
 #include <fcntl.h>
 
+/* Globally imported variables */
+extern GtkWidget *main_window;
+
 /* Local variables */
 static gint preffd;
 
@@ -217,123 +224,178 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
   return filelist;
 }
 
-static GList *get_metadata_for_dirlist(GList *inlist, listtype_t listtype)
+static GList *get_metadata_for_dirlist(GList *inlist, 
+				       listtype_t listtype, 
+				       dirfill_thread_arg_t *args)
 {
   GList *metalist = NULL;
-  GList *filelist = inlist;
+  GList *filelist;
+  guint nofiles, currentfile;
+  gboolean use_dialog = FALSE;
+  gboolean cancelled = FALSE;
 
-  while (filelist)
-    {
-      metadata_t *meta;
-      gchar *path;
-      gchar *data;
-      gchar *tmp;
-      guint size;
-      gint i;
-
-      meta = new_metadata_t();
-      /* Full path -> ID column */
-      path = filelist->data;
-      meta->path = filename_toutf8(path);
-      if (meta->path == NULL) {
-	g_print("%s created a NULL value in conversion to UTF-8!\n", path);
-      }
-      /* If filename is ISO-8859-1 convert it by force! */
+  /* Count the number of files */
+  if (args != NULL) {
+    use_dialog = TRUE;
+    filelist = inlist;
+    nofiles = 0;
+    while (filelist) {
+      nofiles ++;
       filelist = filelist->next;
-      meta->size = GPOINTER_TO_UINT(filelist->data);
-      tmp = g_path_get_basename(path);
-      meta->filename = filename_toutf8(tmp);
-      g_free(tmp);
-      if (listtype == HDDATA_LIST) {
-	/* For data files, provide basic data only */
-	add_metadata_to_model(meta, listtype);
-	// FIXME: fails with size 0 files doesn't it?
-      } else {
-	/* For music files, fill in row by using ID3lib and friends */
-	if (meta->size != 0) {
-	  /* Find codec first */
-	  static gchar codec[4];
-	  
-	  strcpy(codec, meta->path+strlen(meta->path)-3);
-	  g_strup(codec);
-	  meta->codec = g_strdup(codec);
-	  if (!strcmp(meta->codec,"MP3")) {
-	    /* Gets the ID3 tag if any */
-	    get_tag_for_mp3file(meta);
-	    /* Scans the mp3 frames to calculate play length */
-	    if ((meta->length == NULL) && (meta->path != NULL)) {
-	      // g_print("Calling length_from_file()...\n");
-	      gchar *tmppath = filename_fromutf8(meta->path);
-	      meta->length = length_from_file(tmppath, meta->size);
-	      g_free(tmppath);
-	    }
-	  }
-	  if (!strcmp(meta->codec,"WAV"))
-	    {} /* FIXME: learn how to read metadata from WAV files */
-	  if (!strcmp(meta->codec,"WMA")) {
-	    get_tag_for_wmafile(meta);
-	  }
-	  /* 
-	   * Gather additional information from the path and file name
-	   * if this preference is set. 
-	   */
-	  if (get_prefs_detect_metadata()) {
-	    info_from_path(meta);
-	  }
-	  
-	  /* At last pad any empty fields to be <unknown>
-	   * (the way playcenter does it) */
-
-	  if (meta->artist == NULL)
-	    meta->artist = g_strdup("<Unknown>");
-	  if (meta->title == NULL)
-	    meta->title = g_strdup("<Unknown>");
-	  if (meta->album == NULL)
-	    meta->album = g_strdup("<Unknown>");
-	  if (meta->codec == NULL)
-	    meta->codec = g_strdup("<Unknown>");
-	  if (meta->genre == NULL)
-	    meta->genre = g_strdup("<Unknown>");
-	  if (meta->length == NULL)
-	    meta->length = g_strdup("<Unknown>");
-	  if (meta->filename == NULL) {
-	    tmp = g_path_get_basename(path);
-	    meta->filename = filename_toutf8(tmp);
-	    g_free(tmp);
-	  }
-	} else {
-	  if (meta->path == NULL) {
-	    meta->artist = g_strdup("<Unknown>");
-	  } else {
-	    /* For directories: copy dirname to Artist field */
-	    gchar **tmpv;
-	    gint i;
-	    
-	    tmpv = g_strsplit(meta->path, G_DIR_SEPARATOR_S, 0);
-	    /* Find last component */
-	    for (i = 0; tmpv[i] != NULL; i++) {};
-	    if (i >= 1)
-	      meta->artist = g_strdup(tmpv[i-1]);
-	    g_strfreev(tmpv);
-	  }
-	}
-      }
-      /* Add this to the metadata list */
-      metalist = g_list_append(metalist, meta);
       filelist = filelist->next;
     }
+    currentfile = 0;
+  }
+  
+  filelist = inlist;
+  while (filelist && !cancelled) {
+    metadata_t *meta;
+    gchar *path;
+    gchar *data;
+    gchar *tmp;
+    guint size;
+    gint i;
+    
+    meta = new_metadata_t();
+    /* Full path -> ID column */
+    path = filelist->data;
+    meta->path = filename_toutf8(path);
+    if (meta->path == NULL) {
+      g_print("%s created a NULL value in conversion to UTF-8!\n", path);
+    }
+    /* If filename is ISO-8859-1 convert it by force! */
+    filelist = filelist->next;
+    meta->size = GPOINTER_TO_UINT(filelist->data);
+    tmp = g_path_get_basename(path);
+    meta->filename = filename_toutf8(tmp);
+    g_free(tmp);
+    if (use_dialog) {
+      gdk_threads_enter();
+      tmp = g_strdup_printf("File %d of %d", currentfile, nofiles);
+      /* g_print("File: %s\n", meta->filename); */
+      currentfile ++;
+      gtk_label_set_text(GTK_LABEL(args->label), tmp);
+      gdk_threads_leave();
+    }
+    if (listtype == HDDATA_LIST) {
+      /* For data files, provide basic data only 
+       * nothing additional is needed.
+       */
+    } else {
+      /* For music files, fill in row by using libid3tag and friends */
+      if (meta->size != 0) {
+	/* Find codec first */
+	static gchar codec[4];
+	
+	strcpy(codec, meta->path+strlen(meta->path)-3);
+	g_strup(codec);
+	meta->codec = g_strdup(codec);
+	if (!strcmp(meta->codec,"MP3")) {
+	  /* Gets the ID3 tag if any */
+	  get_tag_for_mp3file(meta);
+	  /* Scans the mp3 frames to calculate play length */
+	  if ((meta->length == NULL) && (meta->path != NULL)) {
+	    // g_print("Calling length_from_file()...\n");
+	    gchar *tmppath = filename_fromutf8(meta->path);
+	    meta->length = length_from_file(tmppath, meta->size);
+	    g_free(tmppath);
+	  }
+	}
+	if (!strcmp(meta->codec,"WAV")) {
+	  /* FIXME: learn how to read metadata from WAV files */
+	  get_tag_for_wavfile(meta);
+	  if (meta->length == NULL) {
+	    meta->length = strdup("0:01");
+	  }
+	}
+	if (!strcmp(meta->codec,"WMA")) {
+	  get_tag_for_wmafile(meta);
+	}
+	/* 
+	 * Gather additional information from the path and file name
+	 * if this preference is set. 
+	 */
+	if (get_prefs_detect_metadata()) {
+	  info_from_path(meta, TRUE);
+	} else {
+	  info_from_path(meta, FALSE);
+	}
+	
+	/* At last pad any empty fields to be <unknown>
+	 * (the way playcenter does it) */
+	
+	if (meta->artist == NULL)
+	  meta->artist = g_strdup("<Unknown>");
+	if (meta->title == NULL)
+	  meta->title = g_strdup("<Unknown>");
+	if (meta->album == NULL)
+	  meta->album = g_strdup("<Unknown>");
+	if (meta->codec == NULL)
+	  meta->codec = g_strdup("<Unknown>");
+	if (meta->genre == NULL)
+	  meta->genre = g_strdup("<Unknown>");
+	if (meta->length == NULL)
+	  meta->length = g_strdup("<Unknown>");
+	if (meta->filename == NULL) {
+	  tmp = g_path_get_basename(path);
+	  meta->filename = filename_toutf8(tmp);
+	  g_free(tmp);
+	}
+      } else {
+	if (meta->path == NULL) {
+	  meta->artist = g_strdup("<Unknown>");
+	} else {
+	  /* For directories: copy dirname to Artist field */
+	  gchar **tmpv;
+	  gint i;
+	  
+	  tmpv = g_strsplit(meta->path, G_DIR_SEPARATOR_S, 0);
+	  /* Find last component */
+	  for (i = 0; tmpv[i] != NULL; i++) {};
+	  if (i >= 1)
+	    meta->artist = g_strdup(tmpv[i-1]);
+	  g_strfreev(tmpv);
+	}
+      }
+    }
+    /* Add this to the metadata list */
+    metalist = g_list_append(metalist, meta);
+    /* Update progress bar */
+    if (use_dialog) {
+      gfloat fraction = ((gdouble) currentfile / (gdouble) nofiles);
+      gdk_threads_enter();
+      gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(args->progress), fraction);
+      gdk_threads_leave();
+      cancelled = args->cancelled;
+    }
+    filelist = filelist->next;
+  }
   return metalist;
 }
 
-GList *get_metadata_dir(listtype_t listtype, gchar *path)
+/**
+ * @param args if this and the following parameter is null the call is assumed to be 
+ *        non-threaded, else it is updated and handled as if called from a thread.
+ *        Else it is assumed to be called from a single-threaded context and will not
+ *        display update information.
+ */
+GList *get_metadata_dir(listtype_t listtype, 
+			gchar *path, 
+			dirfill_thread_arg_t *args)
 {
   GList *metalist;
   GList *filelist;
   GList *tmp;
 
   /* If it is for the data view, get for ALL files */
+  if (args != NULL) {
+    gdk_threads_enter();
+    gtk_label_set_text(GTK_LABEL(args->label), _("Getting file list from filesystem"));
+    gdk_threads_leave();
+  }
   filelist = get_directory_listing(path, (listtype == HDDATA_LIST));
-  metalist = get_metadata_for_dirlist(filelist, listtype);
+  /* This is the heavy part */
+  metalist = get_metadata_for_dirlist(filelist, listtype, args);
   /* Free the filelist, not used any more */
   tmp = filelist;
   while (tmp) {
@@ -345,23 +407,137 @@ GList *get_metadata_dir(listtype_t listtype, gchar *path)
   return metalist;
 }
 
-void fill_in_dir(listtype_t listtype, gchar *path)
+gpointer dirfill_thread(gpointer thread_args)
 {
-  GList *metalist = get_metadata_dir(listtype, path);
+  dirfill_thread_arg_t *args = (dirfill_thread_arg_t *) thread_args;
+  GList *metalist;
   GList *tmp;
 
-  recreate_list_store(listtype);
+  /* This is time consuming */
+  metalist = get_metadata_dir(args->listtype, args->path, args);
+
+  /* This will interfere with other threads. */
+  gdk_threads_enter();
+  gtk_label_set_text(GTK_LABEL(args->label), _("Adding metadata to view"));
+  recreate_list_store(args->listtype);
   tmp = metalist;
   while (tmp) {
     metadata_t *meta;
     
     meta = (metadata_t *) tmp->data;
-    add_metadata_to_model(meta, listtype);
+    add_metadata_to_model(meta, args->listtype);
     tmp = tmp->next;
     destroy_metadata_t(meta);
   }
+  view_and_sort_list_store(args->listtype);
+  gtk_widget_destroy(args->dialog);
+  gdk_threads_leave();
   g_list_free(metalist);
-  view_and_sort_list_store(listtype);
+  g_free(args->path);
+  g_free(args);
+}
+
+/**
+ * This callback function will cancel the ongoing
+ * fill-in of metadata for a certain thread.
+ */
+#if GTK_CHECK_VERSION(2,4,0)
+void swapped_cancel_fillin_callback(gpointer *data)
+{
+  dirfill_thread_arg_t *args = (dirfill_thread_arg_t *) data;  
+
+  g_print("Cancelled scan of %s!\n", args->path);
+  args->cancelled = TRUE;
+}
+#else
+void cancel_fillin_callback(GtkWidget *widget, gpointer *data)
+{
+  dirfill_thread_arg_t *args = (dirfill_thread_arg_t *) data;  
+
+  g_print("Cancelled scan of %s!\n", args->path);
+  args->cancelled = TRUE;
+}
+#endif
+
+/**
+ * This will fire up a separate thread which retrieves
+ * the metadata for a certain host-side directory. Both
+ * tracks and common data files.
+ * @param listtype the list pane to retrieve. Must be one
+ *        of the harddisk-refering panes.
+ * @param path the path to retrieve metadata from.
+ */
+void fill_in_dir(listtype_t listtype, gchar *path)
+{
+  dirfill_thread_arg_t *dirfill_thread_args;
+  GtkWidget *label1;
+  GtkWidget *dialog, *scanlabel, *metadata_progress_bar, *separator;
+  dirfill_thread_args = (dirfill_thread_arg_t *) g_malloc(sizeof(dirfill_thread_arg_t));
+
+  // Create a dialog...
+#if GTK_CHECK_VERSION(2,4,0)
+  dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
+				   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_INFO,
+				   GTK_BUTTONS_CLOSE,
+				   (listtype == HDDATA_LIST) ? _("Getting file metadata") : _("Getting track metadata"));
+  gtk_window_set_title (GTK_WINDOW (dialog), (listtype == HDDATA_LIST) ? _("Getting file metadata") : _("Getting track metadata"));
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+  gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
+  //gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  g_signal_connect_swapped(GTK_OBJECT(dialog),
+		   "delete_event",
+		   G_CALLBACK(swapped_cancel_fillin_callback),
+		   (gpointer) dirfill_thread_args);
+  g_signal_connect_swapped(GTK_OBJECT(dialog), 
+		   "response",
+		   G_CALLBACK(swapped_cancel_fillin_callback), 
+		   (gpointer) dirfill_thread_args);
+  label1 = gtk_label_new(path);
+  scanlabel = gtk_label_new("");
+  metadata_progress_bar = gtk_progress_bar_new();
+  separator = gtk_hseparator_new ();
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scanlabel, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), metadata_progress_bar, TRUE, TRUE, 0);
+  gtk_progress_bar_update(GTK_PROGRESS_BAR(metadata_progress_bar), (gfloat) 0);
+  gtk_widget_show_all(dialog);
+#else
+  dialog = gnome_dialog_new((listtype == HDDATA_LIST) ? _("Getting file metadata") : _("Getting track metadata"),
+			    GNOME_STOCK_BUTTON_CANCEL,
+			    NULL);
+  gnome_dialog_button_connect(GNOME_DIALOG(dialog),
+			      0,
+			      G_CALLBACK(cancel_fillin_callback),
+			      (gpointer) dirfill_thread_args);
+  g_signal_connect(GTK_OBJECT(dialog),
+		   "delete_event",
+		   G_CALLBACK(cancel_fillin_callback),
+		   (gpointer) dirfill_thread_args);
+  label1 = gtk_label_new((listtype == HDDATA_LIST) ? _("Getting file metadata") : _("Getting track metadata"));
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
+  label1 = gtk_label_new(path);
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
+  scanlabel = gtk_label_new("");
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), scanlabel, TRUE, TRUE, 0);
+  separator = gtk_hseparator_new ();
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
+  metadata_progress_bar = gtk_progress_bar_new();
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), metadata_progress_bar, TRUE, TRUE, 0);
+  gtk_progress_bar_update(GTK_PROGRESS_BAR(metadata_progress_bar), (gfloat) 0);
+  gtk_widget_show_all(dialog);
+#endif
+  
+  dirfill_thread_args->dialog = dialog;
+  dirfill_thread_args->label = scanlabel;
+  dirfill_thread_args->progress = metadata_progress_bar;
+  dirfill_thread_args->listtype = listtype;
+  dirfill_thread_args->path = g_strdup(path);
+  dirfill_thread_args->cancelled = FALSE;
+    
+  // TODO: check no reentrance while this thread is running...
+  g_thread_create(dirfill_thread,(gpointer) dirfill_thread_args, FALSE, NULL);
 }
 
 gchar *get_current_dir()
@@ -564,7 +740,7 @@ gboolean test_writeable(gchar *path)
   FILE *file;
 
   /* Test to know if we can write into the file */
-  if ((file=fopen(path,"r+"))==NULL ) {
+  if ((file = fopen(path,"r+"))==NULL ) {
     return FALSE;
   }
   fclose(file);
@@ -580,6 +756,36 @@ gboolean create_directory(gchar *path)
   }
 }
 
+void create_dirs(gchar *filename) {
+  struct stat st;
+  gchar **tmpv;
+  gchar *path;
+  gint i, dirs;
+
+  tmpv = g_strsplit(filename, G_DIR_SEPARATOR_S, 0);
+  for (dirs=0; tmpv[dirs]; ++dirs);
+  if (--dirs < 1)
+    goto out;
+  
+  path = g_strdup(tmpv[0]);
+  for (i=0;;) {
+    gchar *tmp;
+    gint ret = 0;
+
+    ret = stat(path, &st);
+    if (ret < 0 || !S_ISDIR(st.st_mode))
+      create_directory(path);
+    if (++i >= dirs)
+      break;
+    tmp = path;
+    path = g_strjoin(G_DIR_SEPARATOR_S, path, tmpv[i], NULL);
+    g_free(tmp);
+  }
+  g_free(path);
+out:
+  g_strfreev(tmpv);
+}
+
 void export_playlistfile(gchar *filename, gchar *formatstring, GList *playlist, gboolean m3ufile, gboolean plsfile)
 {
   gchar *tmpname = filename_toutf8(filename);
@@ -612,7 +818,7 @@ void export_playlistfile(gchar *filename, gchar *formatstring, GList *playlist,
 
     meta = (metadata_t *) tmp->data;
     tmp = g_list_next(tmp);
-    song = compose_metadata_string(meta, formatstring);
+    song = compose_metadata_string(meta, formatstring, FALSE);
     if (m3ufile || plsfile) {
       guint tmplen = mmss_to_seconds(meta->length);
       gchar *tmpfname = compose_filename(meta);
