@@ -23,13 +23,25 @@ Boston, MA 02111-1307, USA.
 #include "playlists.h"
 #include "prefs.h"
 #include "util.h"
+#if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
+#endif
+
+/* This one should be global really */
+GtkWidget *main_window;
 
 /* Local variables */
+#if GTK_CHECK_VERSION(2,6,0)
+static GdkPixbuf *icon_pixbuf;
+#endif
+#if !GTK_CHECK_VERSION(2,4,0)
 static GtkWidget *jukeboxowner_dialog;
 static GtkWidget *jukeboxowner_entry;
 static GtkWidget *select_jukebox_dialog;
 static GtkWidget *jukebox_menu;
+#endif
+// The expose() event will be treated JUST ONCE at the first window creation
+static gboolean justonce = TRUE;
 
 /* Expect these functions */
 void scan_jukebox(gpointer data);
@@ -78,6 +90,7 @@ static GtkWidget *create_jukebox_menu(gchar **jukeboxes)
   return option_menu;
 }
 
+#if !GTK_CHECK_VERSION(2,4,0)
 static GCallback jukebox_ok_signal(GtkButton *button,
 				   gpointer data)
 {
@@ -89,6 +102,7 @@ static GCallback jukebox_ok_signal(GtkButton *button,
     scan_jukebox(NULL);
   }
 }
+#endif
 
 static void create_select_jukebox_dialog(void)
 {
@@ -99,6 +113,40 @@ static void create_select_jukebox_dialog(void)
       int i;
       GtkWidget *label;
       // Build a selection menu
+#if GTK_CHECK_VERSION(2,4,0)
+      GtkWidget *select_jukebox_dialog;
+      GtkWidget *jukebox_menu;
+
+      select_jukebox_dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
+						      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						      GTK_MESSAGE_QUESTION,
+						      GTK_BUTTONS_OK_CANCEL,
+						      _("Select jukebox"));
+      jukebox_menu = create_jukebox_menu(jukeboxes);
+      gtk_window_set_position (GTK_WINDOW(select_jukebox_dialog), GTK_WIN_POS_MOUSE);
+      gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(select_jukebox_dialog)->vbox), 5);
+
+      label = gtk_label_new(_("Select jukebox to operate:"));
+      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(select_jukebox_dialog)->vbox), 
+			 label, TRUE, TRUE, 0);
+      gtk_widget_show(label);
+      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(select_jukebox_dialog)->vbox), 
+			 jukebox_menu, TRUE, TRUE, 0);
+      gtk_widget_show(jukebox_menu);
+      gtk_widget_show_all(select_jukebox_dialog);
+      i = gtk_dialog_run(GTK_DIALOG(select_jukebox_dialog));
+      if (i == GTK_RESPONSE_OK) {
+	guint selection = gtk_option_menu_get_history(GTK_OPTION_MENU(jukebox_menu));
+	gtk_widget_destroy(select_jukebox_dialog);
+	// Select the first jukebox
+	if (jukebox_select(selection)) {
+	  // Initially scan the jukebox
+	  scan_jukebox(NULL);
+	}
+      } else {
+	gtk_widget_destroy(select_jukebox_dialog);
+      }
+#else
       select_jukebox_dialog = gnome_dialog_new(_("Select jukebox"),
 					       GNOME_STOCK_BUTTON_OK,
 					       GNOME_STOCK_BUTTON_CANCEL,
@@ -118,7 +166,7 @@ static void create_select_jukebox_dialog(void)
 			      G_CALLBACK(gtk_widget_destroy),
 			      GTK_OBJECT(select_jukebox_dialog),
 			      0);
-      label = gtk_label_new(_("Select jukebox to operate"));
+      label = gtk_label_new(_("Select jukebox to operate:"));
       gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(select_jukebox_dialog)->vbox), 
 			 label, TRUE, TRUE, 0);
       gtk_widget_show(label);
@@ -127,6 +175,7 @@ static void create_select_jukebox_dialog(void)
       gtk_widget_show(jukebox_menu);
       gtk_widget_show(select_jukebox_dialog);
       g_strfreev(jukeboxes);
+#endif
     } else {
       // Select the first jukebox
       if (jukebox_select(0)) {
@@ -142,10 +191,8 @@ static gint expose(GtkWidget *widget,
 		   GtkWidget *event,
 		   gpointer   data)
 {
-  // This will be treated JUST ONCE at the first window creation
-  static gboolean justonce = TRUE;
-
   if (justonce) {
+    justonce = FALSE;
     if (get_prefs_autoscan()) {
       // This will be treated JUST ONCE at the first window creation
       create_select_jukebox_dialog();
@@ -155,7 +202,6 @@ static gint expose(GtkWidget *widget,
 	jukebox_synchronize_time();
       }
     }
-    justonce = FALSE;
   }
   return(FALSE);
 }
@@ -172,6 +218,23 @@ static void about_box(gpointer data)
   gchar *translator_credits = _("translator_credits");
 
   GtkWidget *about;
+  /*
+   * For a modern GTK+ we remove the use of libgnomeui stuff.
+   */
+#if GTK_CHECK_VERSION(2,6,0)
+  about = gtk_about_dialog_new();
+  gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), PACKAGE);
+  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
+  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), copystring);
+  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), aboutstring);
+  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), "GPL version 2");
+  gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), "http://gnomad2.sourceforge.net/");
+  gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about), "Gnomad Homepage");
+  gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), authors);
+  if (strcmp(translator_credits, "translator_credits")) {
+    gtk_about_dialog_set_translator_credits(GTK_ABOUT_DIALOG(about), translator_credits);
+  }
+#else
   about = gnome_about_new(PACKAGE,
 			  VERSION,
 			  copystring,
@@ -180,6 +243,7 @@ static void about_box(gpointer data)
 			  thanks,
 			  strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 			  NULL);
+#endif
   gtk_widget_show(about);
 }
 
@@ -203,7 +267,29 @@ void scan_jukebox(gpointer data)
   jukebox_locked = TRUE;
   cancel_jukebox_operation = FALSE;
 
-  label = gtk_label_new(_("Retrieveing information from jukebox"));
+#if GTK_CHECK_VERSION(2,4,0)
+  dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
+				   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_INFO,
+				   GTK_BUTTONS_CLOSE,
+				   _("Scanning jukebox library"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Retrieveing metadata from jukebox"));
+  g_signal_connect_object(GTK_OBJECT(dialog),
+			  "delete_event",
+			  G_CALLBACK(cancel_jukebox_operation_click),
+			  NULL,
+			  0);
+  g_signal_connect_object(GTK_OBJECT(dialog), 
+			  "response",
+			  G_CALLBACK(cancel_jukebox_operation_click), 
+			  NULL,
+			  0);
+  /* A blank label for progress messages */
+  label = gtk_label_new("");
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show_all(dialog);
+#else
+  label = gtk_label_new(_("Retrieveing metadata from jukebox"));
   dialog = gnome_dialog_new(_("Scanning jukebox library"),
 			    GNOME_STOCK_BUTTON_CANCEL,
 			    NULL);
@@ -211,9 +297,14 @@ void scan_jukebox(gpointer data)
 			      0,
 			      G_CALLBACK(cancel_jukebox_operation_click),
 			      NULL);
+  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE,
+		      TRUE, 0);
+  /* A blank label for progress messages */
+  label = gtk_label_new("");
   gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE,
 		      TRUE, 0);
   gtk_widget_show_all(dialog);
+#endif
 
   /* A bunch of arguments needed by the scanning thread */
   scan_thread_args.dialog = dialog;
@@ -224,6 +315,7 @@ void scan_jukebox(gpointer data)
   return;
 }
 
+#if !GTK_CHECK_VERSION(2,4,0)
 static void set_owner_ok_button (GtkButton *button,
 				 gpointer data)
 {
@@ -238,6 +330,7 @@ static void set_owner_ok_button (GtkButton *button,
   g_free(owner);
   gtk_widget_destroy(jukeboxowner_dialog);
 }
+#endif
 
 /* A dialog to select the owner */
 static void set_jukeboxowner_dialog(gpointer data)
@@ -246,7 +339,52 @@ static void set_jukeboxowner_dialog(gpointer data)
   gchar *text;
   gchar *ownerstring;
 
+#if GTK_CHECK_VERSION(2,4,0)
+  GtkWidget *jukeboxowner_dialog;
+  GtkWidget *jukeboxowner_entry;
+  gint i;
+  gchar *owner;
 
+  if (!jukebox_selected()) {
+    create_select_jukebox_dialog();
+    return;
+  }
+  jukeboxowner_dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
+						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+						GTK_MESSAGE_QUESTION,
+						GTK_BUTTONS_OK_CANCEL,
+						_("Set owner string"));
+  gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(jukeboxowner_dialog)->vbox), 5);
+  
+  label = gtk_label_new(_("Edit the owner of this jukebox:"));
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(jukeboxowner_dialog)->vbox), 
+		     label, TRUE, TRUE, 0);
+  gtk_widget_show(label);
+  jukeboxowner_entry = gtk_entry_new();
+  ownerstring = jukebox_get_ownerstring();
+  if (ownerstring != NULL)
+    gtk_entry_set_text(GTK_ENTRY(jukeboxowner_entry), ownerstring);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(jukeboxowner_dialog)->vbox), jukeboxowner_entry, TRUE, TRUE, 0);
+  gtk_widget_show(jukeboxowner_entry);
+  gtk_widget_show_all(jukeboxowner_dialog);
+  i = gtk_dialog_run(GTK_DIALOG(jukeboxowner_dialog));
+  if (i == GTK_RESPONSE_OK) {
+    if (jukebox_locked) {
+      create_error_dialog(_("Jukebox busy - change discarded"));
+      return;
+    }
+    owner = g_strdup(gtk_entry_get_text(GTK_ENTRY(jukeboxowner_entry)));
+    jukebox_set_ownerstring(owner);
+    g_free(owner);
+    gtk_widget_destroy(jukeboxowner_dialog);    
+  } else {
+    gtk_widget_destroy(jukeboxowner_dialog);
+  }
+#else
+  if (!jukebox_selected()) {
+    create_select_jukebox_dialog();
+    return;
+  }
   jukeboxowner_dialog = gnome_dialog_new(_("Set owner string"),
 					 GNOME_STOCK_BUTTON_OK,
 					 GNOME_STOCK_BUTTON_CANCEL,
@@ -270,183 +408,25 @@ static void set_jukeboxowner_dialog(gpointer data)
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(jukeboxowner_dialog)->vbox), jukeboxowner_entry, TRUE, TRUE, 0);
   gtk_widget_show(jukeboxowner_entry);
   gtk_widget_show(jukeboxowner_dialog);
+#endif
 }
 
-static void add_to_dialog(GtkWidget *dialog, GtkWidget *thing)
-{
-  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), thing, TRUE, TRUE, 0);
-}
 
 static void information_dialog(gpointer data)
 {
-  GtkWidget *label, *button, *dialog, *hbox;
-  gchar tmp[50];
-#ifdef G_HAVE_GINT64
-  guint64 total, free, used;
-  guint songs, playlists, datafiles;
-#endif
+  GtkWidget *dialog;
 
   if (jukebox_locked) {
     create_error_dialog(_("Cannot view information! Jukebox is busy!"));
     return;
   }
 
-  dialog = gnome_dialog_new(_("Jukebox information"),
-			    GNOME_STOCK_BUTTON_OK,
-			    NULL);
-  gnome_dialog_button_connect_object(GNOME_DIALOG(dialog),
-				     0,
-				     GTK_SIGNAL_FUNC(gtk_widget_destroy),
-				     GTK_OBJECT(dialog));
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Unique device ID:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  label = gtk_label_new(jukebox_get_idstring());
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Firmware revision:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  label = gtk_label_new(jukebox_get_firmware());
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Product name:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  label = gtk_label_new(jukebox_get_prodname());
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Power connected:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  label = gtk_label_new((jukebox_get_power()) ? _("Yes") : _("No"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Jukebox owner:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%s", jukebox_get_ownerstring());
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-#ifdef NJB_HASGETTIME
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Time on jukebox:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  label = gtk_label_new(jukebox_get_time());
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-#endif
-
-#ifdef G_HAVE_GINT64
-  jukebox_getusage(&total, &free, &used, &songs, &playlists, &datafiles);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Total bytes on disk:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%Lu (%Lu MB)", 
-	  total, (guint64) total/G_GINT64_CONSTANT(1048576));
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Total free bytes available:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%Lu (%Lu MB)", 
-	  free, (guint64) free/G_GINT64_CONSTANT(1048576));
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Total bytes used:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%Lu (%Lu MB)", 
-	  used, (guint64) used/G_GINT64_CONSTANT(1048576));
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Number of songs:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%lu", songs);
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Number of playlists:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%lu", playlists);
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new (_("Number of datafiles:"));
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-  add_empty_hbox (hbox);
-  sprintf(tmp, "%lu", datafiles);
-  label = gtk_label_new(tmp);
-  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show(label);
-  add_to_dialog(dialog, hbox);
-  gtk_widget_show (hbox);
-#endif
-
-  gtk_widget_show(dialog);  
+  dialog = jukebox_get_deviceinfo_dialog();
+  if (dialog != NULL) {
+    gtk_widget_show(dialog);  
+  } else {
+    create_error_dialog(_("Cannot retrieve jukebox information!"));
+  }
 }
 
 /***********************************************************************************/
@@ -456,8 +436,9 @@ int main(int argc,
 	 char *argv[])
 {
   const gchar cProgramName[]= PACKAGE " " VERSION;
-  GtkWidget *window;
+#if !GTK_CHECK_VERSION(2,6,0)
   GnomeProgram *gnomad;
+#endif
   /* For the menus */
   GtkWidget *menu;
   GtkWidget *menu_bar;
@@ -477,6 +458,7 @@ int main(int argc,
   extern char *optarg;
   gint opt;
   gint i;
+  GError *icon_load_error;
 
   /* Binds for internationalization (i18n) */
   bindtextdomain (GETTEXT_PACKAGE, GNOMADLOCALEDIR);
@@ -485,12 +467,14 @@ int main(int argc,
 
   g_thread_init(NULL);
   gdk_threads_init();
-  
+
+#if GTK_CHECK_VERSION(2,6,0)
+  gtk_init(&argc, &argv);
+#else
   gnomad = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
 			      GNOME_PARAM_POPT_TABLE, NULL, 
 			      GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
-  // gnome_init(PACKAGE, VERSION, argc, argv);
-
+#endif
   /* Parse extra arguments */
   gnomad_debug = 0;
   while ((opt = getopt(argc, argv, "D:")) != -1 ) {
@@ -512,33 +496,46 @@ int main(int argc,
     gdk_color_black(cmap, &darkblue);
   }
 
-  window = gnome_app_new(PACKAGE, cProgramName);
-  gtk_window_set_wmclass(GTK_WINDOW(window), PACKAGE, "main");
-  gtk_widget_set_usize(GTK_WIDGET(window), 640, 480);
+#if GTK_CHECK_VERSION(2,6,0)
+  main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (main_window), cProgramName);
+  icon_pixbuf = gdk_pixbuf_new_from_file(PIXMAPSDIR "/gnomad2-logo.png", &icon_load_error);
+  if (icon_pixbuf != NULL) {
+    gtk_window_set_default_icon(icon_pixbuf);
+  }
+#else
+  main_window = gnome_app_new(PACKAGE, cProgramName);
+#endif
+  gtk_window_set_wmclass(GTK_WINDOW(main_window), PACKAGE, "main");
+  gtk_widget_set_usize(GTK_WIDGET(main_window), 640, 480);
   
-  g_signal_connect(GTK_OBJECT(window), 
+  g_signal_connect(GTK_OBJECT(main_window), 
 		   "delete_event",
 		   G_CALLBACK(delete),
 		   NULL);
 
-  g_signal_connect(GTK_OBJECT(window), 
+  g_signal_connect(GTK_OBJECT(main_window), 
 		   "expose_event",
 		   G_CALLBACK(expose),
 		   NULL);
   
-  gtk_container_set_border_width (GTK_CONTAINER (window), 0);
+  gtk_container_set_border_width (GTK_CONTAINER (main_window), 0);
 
   /* Create metadata list stores */
   create_list_stores();
 
   /* Create a GtkAccelGroup and add it to the window. */
   accel_group = gtk_accel_group_new();
-  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+  gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);
 
   /* Stuff everything into a vertical box */
-  vbox=gtk_vbox_new(FALSE, 5);
+  vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
-  gnome_app_set_contents(GNOME_APP(window), vbox);
+#if GTK_CHECK_VERSION(2,6,0)
+  gtk_container_add(GTK_CONTAINER(main_window), vbox);
+#else
+  gnome_app_set_contents(GNOME_APP(main_window), vbox);
+#endif
   gtk_widget_show(vbox);
 
   /* Create the menu bar */
@@ -660,7 +657,7 @@ int main(int argc,
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), root_menu);
   
-  gtk_widget_show(window);
+  gtk_widget_show(main_window);
 
   gdk_threads_enter();
   gtk_main();
