@@ -35,7 +35,7 @@ static id3_utf8_t *charset_to_utf8(const id3_latin1_t *str)
 {
   id3_utf8_t *tmp;
   
-  tmp = g_convert(str,-1,"UTF-8","ISO-8859-1",NULL,NULL,NULL);
+  tmp = (id3_utf8_t *) g_convert((gchar *) str,-1,"UTF-8","ISO-8859-1",NULL,NULL,NULL);
   return (id3_utf8_t *) tmp;
 }
 
@@ -43,7 +43,7 @@ static id3_latin1_t *charset_from_utf8(const id3_utf8_t *str)
 {
   id3_latin1_t *tmp;
 
-  tmp = g_convert(str,-1,"ISO-8859-1","UTF-8",NULL,NULL,NULL);
+  tmp = (id3_latin1_t *) g_convert((gchar *) str,-1,"ISO-8859-1","UTF-8",NULL,NULL,NULL);
   return (id3_latin1_t *) tmp;
 }
 
@@ -321,13 +321,13 @@ getFrameText(struct id3_tag *tag, char *frame_name)
        whatever coding system they are using into it, so we use
        charset_to_utf8() to convert to utf8 */
     id3_latin1_t *raw = id3_ucs4_latin1duplicate(string);
-    utf8 = charset_to_utf8(raw);
+    utf8 = (gchar *) charset_to_utf8(raw);
     g_free (raw);
   } else {
     /* Standard unicode is being used -- we won't have to worry
        about charsets then. */
     // g_print("This frame is a Unicode frame!\n");
-    utf8 = id3_ucs4_utf8duplicate (string);
+    utf8 = (gchar *) id3_ucs4_utf8duplicate (string);
   }
   // g_print("Found tag: %s, value: %s\n", frame_name, utf8);
   return utf8;
@@ -399,14 +399,14 @@ setFrameText(struct id3_tag *tag,
     if (index != -1) {
       /* valid genre -- simply store the genre number */
       gchar *tmp = g_strdup_printf("%d", index);
-      ucs4 = id3_latin1_ucs4duplicate (tmp);
+      ucs4 = id3_latin1_ucs4duplicate ((id3_latin1_t *) tmp);
       g_free (tmp);
     } else {
       /* oups -- not a valid genre -- save the entire genre string */
       if (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1) {
 	/* we read 'ISO_8859_1' to stand for 'any locale
 	   charset' -- most programs seem to work that way */
-	id3_latin1_t *raw = charset_from_utf8(data);
+	id3_latin1_t *raw = charset_from_utf8((id3_utf8_t *) data);
 	ucs4 = id3_latin1_ucs4duplicate (raw);
 	g_free (raw);
       } else {
@@ -421,7 +421,7 @@ setFrameText(struct id3_tag *tag,
     if (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1) {
       /* we read 'ISO_8859_1' to stand for 'any locale charset'
 	 -- most programs seem to work that way */
-      id3_latin1_t *raw = charset_from_utf8(data);
+      id3_latin1_t *raw = charset_from_utf8((id3_utf8_t *) data);
       ucs4 = id3_latin1_ucs4duplicate (raw);
       g_free (raw);
     } else {
@@ -698,11 +698,11 @@ remove_tag_from_mp3file(gchar *path)
 	tag[0] == 'I' &&
 	tag[1] == 'D' &&
 	tag[2] == '3') {
-      // g_print("Found ID3v2 tag header...\n");
+      // g_print("Found ID3v2 tag header...");
       // Get tag length from the tag
       header_taglength = (tag[6] << 24) + (tag[7] << 16) + (tag[8] << 8) + tag[9];
       // This needs some heuristics... first scan until only zeroes are found.
-      // while ()
+      // g_print(" given size %d (%08x)\n", header_taglength, header_taglength);
       n = read(f1,buffer,bufsiz);
       if (n > 4) {
 	guint m = 0;
@@ -723,7 +723,9 @@ remove_tag_from_mp3file(gchar *path)
 	  }
 	  m += 8;
 	}
-	// g_print("Found zeros\n");
+	if (foundzeros == TRUE) {
+	  // g_print("Found zeros\n");
+	}
 
 	// Then look for first non-zero
 	while (!founddata && (m < n)) {
@@ -745,7 +747,7 @@ remove_tag_from_mp3file(gchar *path)
 	if (founddata) {
 	  // g_print("Found data\n");
 	  if (header_taglength != m) {
-	    g_print("Bad header tag! Given length: %d, detected length: %d\n", header_taglength, m);
+	    g_print("remove_tag_from_mp3file(): Bad header tag! Given length: %d, detected length: %d\n", header_taglength, m);
 	  }
 	  // Adjust header taglength
 	  header_taglength = m;
@@ -882,6 +884,11 @@ get_tag_for_mp3file (metadata_t *meta)
 
   /* g_print("Getting tag info for %s...\n", path); */
   fh = id3_file_open(tmppath, ID3_FILE_MODE_READONLY);
+  if (fh == 0) {
+    g_print("get_tag_for_mp3file(), file could not be opened: %s\n", tmppath);
+    g_print("This may be caused by bad locale settings.\n");
+    return;
+  }
   g_free(tmppath);
   tag = id3_file_tag(fh);
 
