@@ -37,6 +37,7 @@ metadata_t *new_metadata_t()
   meta->protect = FALSE;
   meta->filename = NULL;
   meta->path = NULL;
+  meta->folder = NULL;
 }
 
 void destroy_metadata_t(metadata_t *meta)
@@ -55,6 +56,8 @@ void destroy_metadata_t(metadata_t *meta)
     g_free(meta->filename);
   if (meta->path != NULL)
     g_free(meta->path);
+  if (meta->folder != NULL)
+    g_free(meta->folder);
   g_free(meta);
 }
 
@@ -81,6 +84,8 @@ metadata_t *clone_metadata_t(metadata_t *meta)
     clone->filename = g_strdup(meta->filename);
   if (meta->path != NULL)
     clone->path = g_strdup(meta->path);
+  if (meta->folder != NULL)
+    clone->folder = g_strdup(meta->folder);
   return clone;
 }
 
@@ -99,10 +104,23 @@ void dump_metadata_t(metadata_t *meta)
   g_print("Protected: %s\n", meta->protect ? "YES" : "NO");
   g_print("Filename:  %s\n", meta->filename);
   g_print("Path:      %s\n", meta->path);
+  g_print("Folder:    %s\n", meta->folder);
 }
 
-gchar *compose_metadata_string(metadata_t *meta, gchar *template)
+static void strip_filename_illegal_chars(gchar *str)
 {
+  guint i;
+
+  for (i = 0; i < strlen(str); i++) {
+    if (str[i] == G_DIR_SEPARATOR) {
+      str[i] = '_';
+    }
+  }
+}
+
+gchar *compose_metadata_string(metadata_t *meta, gchar *template, gboolean filesystemsafe)
+{
+  gchar *tmp;
   gchar *tmpyear;
   gchar *tmptrackno;
   gchar *returnname;
@@ -113,22 +131,46 @@ gchar *compose_metadata_string(metadata_t *meta, gchar *template)
 
   returnname = g_strdup(template);
 
-  if (meta->artist)
-    returnname = replacestring(returnname, "%a", meta->artist);
-  else
+  if (meta->artist) {
+    tmp = g_strdup(meta->artist);
+    if (filesystemsafe) {
+      strip_filename_illegal_chars(tmp);
+    }
+    returnname = replacestring(returnname, "%a", tmp);
+    g_free(tmp);
+  } else {
     returnname = replacestring(returnname, "%a", "<Unknown>");
-  if (meta->title)
-    returnname = replacestring(returnname, "%t", meta->title);
-  else
+  }
+  if (meta->title) {
+    tmp = g_strdup(meta->title);
+    if (filesystemsafe) {
+      strip_filename_illegal_chars(tmp);
+    }
+    returnname = replacestring(returnname, "%t", tmp);
+    g_free(tmp);
+  } else {
     returnname = replacestring(returnname, "%t", "<Unknown>");
-  if (meta->album)
-    returnname = replacestring(returnname, "%b", meta->album);
-  else
+  }
+  if (meta->album) {
+    tmp = g_strdup(meta->album);
+    if (filesystemsafe) {
+      strip_filename_illegal_chars(tmp);
+    }
+    returnname = replacestring(returnname, "%b", tmp);
+    g_free(tmp);
+  } else {
     returnname = replacestring(returnname, "%b", "<Unknown>");
-  if (meta->genre)
-    returnname = replacestring(returnname, "%g", meta->genre);
-  else
+  }
+  if (meta->genre) {
+    tmp = g_strdup(meta->genre);
+    if (filesystemsafe) {
+      strip_filename_illegal_chars(tmp);
+    }
+    returnname = replacestring(returnname, "%g", tmp);
+    g_free(tmp);
+  } else {
     returnname = replacestring(returnname, "%g", "<Unknown>");
+  }
   tmptrackno = g_strdup_printf("%02lu", meta->trackno);
   if (tmptrackno) {
     returnname = replacestring(returnname, "%n", tmptrackno);
@@ -167,6 +209,7 @@ metadata_t *get_metadata_from_model(GtkTreeModel *model,
 		     TRACKLIST_PROTECTED_COLUMN, &meta->protect,
 		     TRACKLIST_FILENAME_COLUMN, &meta->filename,
 		     TRACKLIST_ID_COLUMN, &meta->path,
+		     TRACKLIST_FOLDER_COLUMN, &meta->folder,
 		     -1);
   return meta;
 }
@@ -179,6 +222,7 @@ metadata_t *get_data_metadata_from_model(GtkTreeModel *model,
 		     FILELIST_FILENAME_COLUMN, &meta->filename,
 		     FILELIST_SIZE_COLUMN, &meta->size,
 		     FILELIST_ID_COLUMN, &meta->path,
+		     FILELIST_FOLDER_COLUMN, &meta->folder,
 		     -1);
   return meta;
 }
@@ -209,6 +253,7 @@ void add_metadata_to_model(metadata_t *meta, listtype_t listtype)
 			TRACKLIST_PROTECTED_COLUMN, meta->protect,
 			TRACKLIST_FILENAME_COLUMN, meta->filename,
 			TRACKLIST_ID_COLUMN, meta->path,
+			TRACKLIST_FOLDER_COLUMN, meta->folder,
 			-1);
   }
 
@@ -223,6 +268,7 @@ void add_metadata_to_model(metadata_t *meta, listtype_t listtype)
 			FILELIST_FILENAME_COLUMN, meta->filename,
 			FILELIST_SIZE_COLUMN, meta->size,
 			FILELIST_ID_COLUMN, meta->path,
+			FILELIST_FOLDER_COLUMN, meta->folder,
 			-1);
   }
 }
@@ -416,7 +462,8 @@ static GtkListStore *create_hdliststore()
 				    G_TYPE_INT,
 				    G_TYPE_BOOLEAN,
 				    G_TYPE_STRING,
-				    G_TYPE_STRING /*ID*/
+				    G_TYPE_STRING, /*ID*/
+				    G_TYPE_STRING /* Folder */
 				    );
   return hdliststore;
 }
@@ -437,7 +484,8 @@ static GtkListStore *create_jbliststore()
 				    G_TYPE_INT,
 				    G_TYPE_BOOLEAN,
 				    G_TYPE_STRING,
-				    G_TYPE_STRING /*ID*/
+				    G_TYPE_STRING, /*ID*/
+				    G_TYPE_STRING /* Folder */
 				    );
   return jbliststore;
 }
@@ -449,7 +497,8 @@ static GtkListStore *create_hddataliststore()
   hddataliststore = gtk_list_store_new (FILELIST_N_COLUMNS,
 					G_TYPE_STRING, /*Filename */
 					G_TYPE_INT, /* Size */
-					G_TYPE_STRING /*ID*/
+					G_TYPE_STRING, /*ID*/
+					G_TYPE_STRING /* Folder */
 				    );
   return hddataliststore;
 }
@@ -461,7 +510,8 @@ static GtkListStore *create_jbdataliststore()
   jbdataliststore = gtk_list_store_new (FILELIST_N_COLUMNS,
 					G_TYPE_STRING, /*Filename */
 					G_TYPE_INT, /* Size */
-					G_TYPE_STRING /*ID*/
+					G_TYPE_STRING, /*ID*/
+					G_TYPE_STRING /* Folder */
 				    );
   return jbdataliststore;
 }
@@ -570,10 +620,10 @@ static gint hdlist_compare_func (GtkTreeModel *model,
  * the files in the harddisk list view. (It is not used at all
  * for the jukebox view.)
  */
-static gint hddatalist_compare_func (GtkTreeModel *model,
-				     GtkTreeIter *a,
-				     GtkTreeIter *b,
-				     gpointer user_data)
+static gint datalist_compare_func (GtkTreeModel *model,
+				   GtkTreeIter *a,
+				   GtkTreeIter *b,
+				   gpointer user_data)
 {
   GType coltype;
   gint retval = 0;
@@ -694,7 +744,7 @@ void view_and_sort_list_store(listtype_t listtype)
     for (i = FILELIST_FILENAME_COLUMN; i < FILELIST_FILENAME_COLUMN + FILELIST_N_COLUMNS; i++) {
       gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(data_widgets.hdliststore),
 				      i,
-				      hddatalist_compare_func,
+				      datalist_compare_func,
 				      NULL,
 				      NULL);
     }
@@ -702,6 +752,13 @@ void view_and_sort_list_store(listtype_t listtype)
 					 FILELIST_FILENAME_COLUMN,
 					 GTK_SORT_ASCENDING);
   } else if (listtype == JBDATA_LIST) {
+    for (i = FILELIST_FILENAME_COLUMN; i < FILELIST_FILENAME_COLUMN + FILELIST_N_COLUMNS; i++) {
+      gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(data_widgets.jbliststore),
+				      i,
+				      datalist_compare_func,
+				      NULL,
+				      NULL);
+    }
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(data_widgets.jbliststore),
 					 FILELIST_FILENAME_COLUMN,
 					 GTK_SORT_ASCENDING);
