@@ -33,6 +33,7 @@ typedef struct
   gboolean protect;
   gchar *filename;
   gchar *path;
+  gchar *folder;
 } metadata_t;
 
 enum
@@ -49,6 +50,7 @@ enum
     TRACKLIST_PROTECTED_COLUMN,
     TRACKLIST_FILENAME_COLUMN,
     TRACKLIST_ID_COLUMN, /* Hidden */
+    TRACKLIST_FOLDER_COLUMN, /* Hidden */
     TRACKLIST_N_COLUMNS
   };
 
@@ -68,6 +70,7 @@ enum
     FILELIST_FILENAME_COLUMN,
     FILELIST_SIZE_COLUMN,
     FILELIST_ID_COLUMN, /* Hidden */
+    FILELIST_FOLDER_COLUMN, /* Hidden */
     FILELIST_N_COLUMNS
   };
 
@@ -84,7 +87,7 @@ metadata_t *new_metadata_t();
 void destroy_metadata_t(metadata_t *meta);
 metadata_t *clone_metadata_t(metadata_t *meta);
 void dump_metadata_t(metadata_t *meta);
-gchar *compose_metadata_string(metadata_t *meta, gchar *template);
+gchar *compose_metadata_string(metadata_t *meta, gchar *template, gboolean filesystemsafe);
 metadata_t *get_metadata_from_model(GtkTreeModel *model, 
 				    GtkTreeIter *iter);
 metadata_t *get_data_metadata_from_model(GtkTreeModel *model, 
