@@ -631,7 +631,7 @@ static int mpg123_get_xing_header(xing_header_t * xing, unsigned char *buf)
 	buf += 9;
     }
   
-  if (strncmp(buf, "Xing", 4))
+  if (strncmp((char *) buf, "Xing", 4))
     return 0;
   buf += 4;
   
@@ -901,7 +901,7 @@ static guint get_track_time_file(FILE * file)
       len = ftell(file);
       fseek(file, -128, SEEK_END);
       fread(tmp, 1, 3, file);
-      if (!strncmp(tmp, "TAG", 3))
+      if (!strncmp((char *) tmp, "TAG", 3))
 	len -= 128;
       return ((guint) ((guint)(len / bpf) * tpf * 1000));
     }
