@@ -483,14 +483,31 @@ static GCallback playlist_delete_response(gpointer data)
 {
   guint plid;
 
+  if (jukebox_locked) {
+    create_error_dialog(_("Jukebox busy"));
+    return;
+  }
+  if (request_confirmation_dialog(_("Really delete selected playlist?"))) {
+    plid = get_playlistid_from_selection();
+    jukebox_delete_playlist(plid);
+    /* g_print("Deleting playlist %lu...\n", plid); */
+    remove_selected_row();
+  }
+}
+
+/* Response to the Shuffle playlist popup menu function */
+static GCallback playlist_shuffle_response(gpointer data)
+{
+  guint plid;
+  guint newplaylist;
+
   if (jukebox_locked) {
     create_error_dialog(_("Jukebox busy"));
     return;
   }
   plid = get_playlistid_from_selection();
-  jukebox_delete_playlist(plid);
-  /* g_print("Deleting playlist %lu...\n", plid); */
-  remove_selected_row();
+  newplaylist = jukebox_randomize_playlist(plid, playlist_widgets.pltreestore);
+  /* g_print("Shuffeling playlist %lu...\n", plid); */
 }
 
 /* Response to the Delete track popup menu function */
@@ -507,13 +524,16 @@ static GCallback node_delete_response(gpointer data)
     return;
   }
 
-  plid = get_playlistid_from_selection();
-  track = get_trackid_from_selection();
-  /* g_print("Trying to delete track %lu from playlist %lu\n", track, plid); */
-  newplaylist = jukebox_delete_track_from_playlist(track, plid);
-  remove_selected_row();
-  /* Change this in all affected places in the liststore */
-  update_playlistid_in_store(plid, newplaylist);
+  if (request_confirmation_dialog(_("Really delete selected tracks?"))) {
+    plid = get_playlistid_from_selection();
+    track = get_trackid_from_selection();
+    /* g_print("Trying to delete track %lu from playlist %lu\n", track, plid); */
+    newplaylist = jukebox_delete_track_from_playlist(track, plid, playlist_widgets.pltreestore);
+    remove_selected_row();
+    /* Change this in all affected places in the liststore */
+    // This should work now.
+    update_playlistid_in_store(plid, newplaylist);
+  }
 }
 
 
@@ -868,6 +888,14 @@ void create_playlist_widgets(GtkWidget *box)
 			  NULL,
 			  0);
   gtk_widget_show (popupmenu_item);
+  popupmenu_item  = gtk_menu_item_new_with_label(_("Shuffle playlist"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(playlistpopupmenu), popupmenu_item);
+  g_signal_connect_object(GTK_OBJECT (popupmenu_item), 
+			  "activate",
+			  G_CALLBACK(playlist_shuffle_response), 
+			  NULL,
+			  0);
+  gtk_widget_show (popupmenu_item);
   popupmenu_item  = gtk_menu_item_new_with_label(_("Play playlist"));
   gtk_menu_shell_append(GTK_MENU_SHELL(playlistpopupmenu), popupmenu_item);
   g_signal_connect_object(GTK_OBJECT (popupmenu_item), 
