@@ -87,12 +87,12 @@ parse_content_description(guchar *data, guint size, metadata_t *meta)
 
   if ((title_length > 0) && (meta->title == NULL)) {
     get_ucs2_string(&data[offset], title_length, &meta->title);
-    g_print("Found title: %s\n", meta->title);
+    // g_print("Found title: %s\n", meta->title);
   }
   offset += title_length;
   if ((offset < size) && (author_length > 0) && (meta->artist == NULL)) {
     get_ucs2_string(&data[offset], author_length, &meta->artist);
-    g_print("Found author/artist: %s\n", meta->artist);
+    // g_print("Found author/artist: %s\n", meta->artist);
   }
   // Skip the other fields.
   offset += author_length;
@@ -121,7 +121,7 @@ parse_extended_content_description(guchar *data, guint size, metadata_t *meta)
   // * Descriptor Value           variable     variable        // value for Content Descriptor
   guint descriptors = (data[25] << 8) + data[24];
   guint offset = 26;
-  g_print("%d descriptors\n", descriptors);
+  // g_print("%d descriptors\n", descriptors);
   while ((descriptors > 0) && (offset < size)) {
     guint desc_namelen;
     gchar *desc_name;
@@ -140,7 +140,7 @@ parse_extended_content_description(guchar *data, guint size, metadata_t *meta)
       // Unicode string. Let's look at it...
       gchar *desc_string = NULL;
       get_ucs2_string(&data[offset], desc_datalen, &desc_string);
-      g_print("Descriptor: %s, value=\'%s\'\n", desc_name, desc_string);
+      // g_print("Descriptor: %s, value=\'%s\'\n", desc_name, desc_string);
       if (!strcmp(desc_name, "WM/Genre") || !strcmp(desc_name, "Genre")) {
 	if (meta->genre != NULL) {
 	  g_free(meta->genre);
@@ -176,7 +176,7 @@ parse_extended_content_description(guchar *data, guint size, metadata_t *meta)
 	meta->trackno = desc_value;
       }
     } else {
-      g_print("Skipped descriptor: %s (type=%d,namelen=%lu,datalen=%lu)\n", desc_name, desc_datatype, desc_namelen, desc_datalen);
+      // g_print("Skipped descriptor: %s (type=%d,namelen=%lu,datalen=%lu)\n", desc_name, desc_datatype, desc_namelen, desc_datalen);
     }
     offset += desc_datalen;
     g_free(desc_name);
@@ -209,20 +209,20 @@ parse_file_properties(guchar *data, guint size, metadata_t *meta)
   guint64 playtime = ((guint64) data[71] << 56) + ((guint64) data[70] << 48) + ((guint64) data[69] << 40) + 
     ((guint64) data[68] << 32) + ((guint64) data[67] << 24) + ((guint64) data[66] << 16) + ((guint64) data[65] << 8) + data[64];
 
-  g_print("Flags value %02X, (%d)\n", flags, flags);
-  g_print("Playtime: %llu\n", playtime);
+  // g_print("Flags value %02X, (%d)\n", flags, flags);
+  // g_print("Playtime: %llu\n", playtime);
   if ((flags & 0x0001) == 0) {
     // We have valid values
     gint seconds;
 
     playtime /= 10000000;
     seconds = (gint) playtime;
-    g_print("Play time %d seconds\n", seconds);
+    // g_print("Play time %d seconds\n", seconds);
     if (meta->length == NULL) {
       meta->length = seconds_to_mmss(seconds);
     }
   } else {
-    g_print("Stream had broadcast flag set: length tag is invalid.\n");
+    // g_print("Stream had broadcast flag set: length tag is invalid.\n");
   }
   return;
 }
@@ -274,19 +274,20 @@ get_tag_for_wmafile (metadata_t *meta)
   gchar *tmppath = filename_fromutf8(meta->path);
   gint n;
 
-  g_print("Getting WMA tag info for %s...\n", meta->path);
+  // g_print("Getting WMA tag info for %s...\n", meta->path);
   fd = (gint) open(tmppath, O_RDONLY, 0);
   if (fd < 0) {
+    g_free(tmppath);
     return;
   }
-  g_print("Opened file\n");
+  // g_print("Opened file\n");
   g_free(tmppath);
 
   // Read in some stuff...
   n = read(fd,header,30);
   if (n == 30) {
     // Hardcoded ASF header GUID
-    gchar ASF_header[] = {0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66 ,0xCF, 0x11, 0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C};
+    guchar ASF_header[] = {0x30, 0x26, 0xB2, 0x75, 0x8E, 0x66 ,0xCF, 0x11, 0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C};
 
     if (guid_compare(ASF_header, header)) {
       guint headersize;
@@ -302,7 +303,7 @@ get_tag_for_wmafile (metadata_t *meta)
       }
       headersize = (header[19] << 24) + (header[18] << 16) + (header[17] << 8) + header[16] - 30;
       objects = (header[27] << 24) + (header[26] << 16) + (header[25] << 8) + header[24];
-      g_print("Found an ASF header of %lu bytes with %lu objects\n", headersize, objects);
+      // g_print("Found an ASF header of %lu bytes with %lu objects\n", headersize, objects);
       headerobject = g_malloc(headersize);
       n = read(fd,headerobject,headersize);
       if (n == headersize) {
@@ -312,7 +313,7 @@ get_tag_for_wmafile (metadata_t *meta)
 	guint current_object = 0;
 	guint offset = 0;
 
-	g_print("Retrieved ASF header\n");
+	// g_print("Retrieved ASF header\n");
 	while (current_object < objects) {
 	  // Available objects
 	  // GUID entrance pattern:
@@ -331,8 +332,7 @@ get_tag_for_wmafile (metadata_t *meta)
 	    {0x40, 0xA4, 0xD0, 0xD2, 0x07, 0xE3, 0xD2, 0x11, 0x97, 0xF0, 0x00, 0xA0, 0xC9, 0x5E, 0xA8, 0x50};
 
 	  memcpy(asf_object_guid, &headerobject[offset], 16);
-	  g_print("Object GUID: %02X-%02X-%02X-%02X...\n", asf_object_guid[0], asf_object_guid[1], 
-		  asf_object_guid[2], asf_object_guid[3]);
+	  // g_print("Object GUID: %02X-%02X-%02X-%02X...\n", asf_object_guid[0], asf_object_guid[1], asf_object_guid[2], asf_object_guid[3]);
 
 	  if (headerobject[offset+20] != 0x00 ||
 	      headerobject[offset+21] != 0x00 ||
@@ -343,20 +343,20 @@ get_tag_for_wmafile (metadata_t *meta)
 	  }
 	  asf_object_size = (headerobject[offset+19] << 24) + (headerobject[offset+18] << 16) + 
 	    (headerobject[offset+17] << 8) + headerobject[offset+16];
-	  g_print("Found object of size %lu\n", asf_object_size);
+	  // g_print("Found object of size %lu\n", asf_object_size);
 	  if (guid_compare(asf_object_guid, File_Properties_Object_GUID)) {
-	    g_print("Found File Properties Object\n");
+	    // g_print("Found File Properties Object\n");
 	    parse_file_properties(&headerobject[offset], asf_object_size, meta);
 	  } else if (guid_compare(asf_object_guid, Stream_Properties_Object_GUID)) {
-	    g_print("Found Stream Properties Object\n");
+	    // g_print("Found Stream Properties Object\n");
 	  } else if (guid_compare(asf_object_guid, Header_Extension_Object_GUID)) {
-	    g_print("Found Header Extension Object\n");
+	    // g_print("Found Header Extension Object\n");
 	    parse_header_extension(&headerobject[offset], asf_object_size, meta);
 	  } else if (guid_compare(asf_object_guid, Content_Description_Object_GUID)) {
-	    g_print("Found a Content Description Object\n");
+	    // g_print("Found a Content Description Object\n");
 	    parse_content_description(&headerobject[offset], asf_object_size, meta);
 	  } else if (guid_compare(asf_object_guid, Extended_Content_Description_Object_GUID)) {
-	    g_print("Found an Extended Content Description Object\n");
+	    // g_print("Found an Extended Content Description Object\n");
 	    parse_extended_content_description(&headerobject[offset], asf_object_size, meta);
 	  }
 	  offset += asf_object_size;
