@@ -101,7 +101,7 @@ static GList *get_files_to_xfer(GList *metalist, GList *appendum) {
 
 	/* g_print("Recursing directory...\n"); */
 	/* Get metalist for directory */
-	inner_metalist = get_metadata_dir(HD_LIST, meta->path);
+	inner_metalist = get_metadata_dir(HD_LIST, meta->path, NULL);
 	/* Call myself */
 	newlist = get_files_to_xfer(inner_metalist, newlist);
 	/* Free the metalist and temppath */
@@ -234,11 +234,13 @@ static GCallback hdmenu_transfer_response(gpointer data)
 /* Response to the "delete" call from the harddisk popup menu */
 static GCallback hdmenu_delete_response(gpointer data)
 {
-  GList *metalist = get_all_metadata_from_selection(HD_LIST);
+  if (request_confirmation_dialog(_("Really delete selected tracks?"))) {
+    GList *metalist = get_all_metadata_from_selection(HD_LIST);
 
-  delete_files(metalist);
-  remove_selected(HD_LIST); // This or fill in directory? This is faster.
-  destroy_metalist(metalist);
+    delete_files(metalist);
+    remove_selected(HD_LIST); // This or fill in directory? This is faster.
+    destroy_metalist(metalist);
+  }
 }
 
 /* Handles descending into a directory in the clist */
@@ -337,12 +339,13 @@ static GCallback jbmenu_play_response(gpointer data)
 /* Response to the "delete" call from the jukebox popup menu */
 static GCallback jbmenu_delete_response(gpointer data)
 {
-  GList *metalist = get_all_metadata_from_selection(JB_LIST);
+  if (request_confirmation_dialog(_("Really delete selected tracks?"))) {
+    GList *metalist = get_all_metadata_from_selection(JB_LIST);
 
-  jukebox_delete_tracks(metalist, playlist_widgets.pltreestore);
-  remove_selected(JB_LIST);
-  destroy_metalist(metalist);
-  
+    jukebox_delete_tracks(metalist, playlist_widgets.pltreestore);
+    remove_selected(JB_LIST);
+    destroy_metalist(metalist);
+  }
 }
 
 static GCallback jblist_mouseevent (GtkWidget      *clist,
@@ -626,7 +629,7 @@ GtkWidget *create_xfer_widgets()
   
   /* The popup menu for the harddisk files */
   popupmenu = gtk_menu_new ();
-  popupmenu_item  = gtk_menu_item_new_with_label (_("Rescan jukebox"));
+  popupmenu_item  = gtk_menu_item_new_with_label (_("Rescan directory metadata"));
   gtk_menu_shell_append (GTK_MENU_SHELL(popupmenu), popupmenu_item);
   g_signal_connect_object(GTK_OBJECT(popupmenu_item),
 			  "activate",
@@ -662,7 +665,7 @@ GtkWidget *create_xfer_widgets()
 
   /* The popup menu for the jukebox files */
   popupmenu = gtk_menu_new ();
-  popupmenu_item  = gtk_menu_item_new_with_label (_("Rescan contents"));
+  popupmenu_item  = gtk_menu_item_new_with_label (_("Rescan jukebox contents"));
   gtk_menu_shell_append (GTK_MENU_SHELL(popupmenu), popupmenu_item);
   g_signal_connect_object(GTK_OBJECT(popupmenu_item),
 			  "activate",
